import arcpy, os, re
from netCDF4 import Dataset

class Toolbox(object):
    def __init__(self):
        self.label =  "Create Hydropower Points"
        self.alias  = "HydroPoints"
        self.tools = [HydroPoints]

class HydroPoints(object):
    def __init__(self):
        self.label = "Create Hydropower Points"
        self.description = "Create points of hydropower potential"

    def getParameterInfo(self):
        geodb = arcpy.Parameter(
            displayName="Working geodatabase (e.g. Default.gdb)",
            name="geodb",
            datatype="DEWorkspace",
            parameterType="Required",
            direction="Input")

        rivers = arcpy.Parameter(
            displayName="HydroSHEDS river network",
            name="rivers",
            datatype="DEFeatureClass",
            parameterType="Required",
            direction="Input")

        elevation = arcpy.Parameter(
            displayName="Digital elevation raster, hole-filled",
            name="elevation",
            datatype="DERasterDataset",
            parameterType="Required",
            direction="Input")

        flowAccu = arcpy.Parameter(
            displayName="Flow accummulation raster",
            name="flowAccu",
            datatype="DERasterDataset",
            parameterType="Required",
            direction="Input")

        discharge = arcpy.Parameter(
            displayName="Raster of discharge, in m3/s",
            name="discharge",
            datatype="DERasterDataset",
            parameterType="Required",
            direction="Input")

        interval = arcpy.Parameter(
            displayName="Interval in metres for point creation",
            name="interval",
            datatype="GPLong",
            parameterType="Required",
            direction="Input")

        points = arcpy.Parameter(
            displayName="Name for new hydro points (will be placed in the specified geodatabase)",
            name="points",
            datatype="GPString",
            parameterType="Required",
            direction="Output")

        parameters = [geodb, rivers, elevation, flowAccu, discharge, interval, points]

        return parameters

    def execute(self, parameters, messages):
        from arcpy import *
        env.overwriteOutput = True
        env.addOutputsToMap = False

        geodb = parameters[0].valueAsText
        rivers = parameters[1].valueAsText
        elevation = parameters[2].valueAsText
        flowAccu = parameters[3].valueAsText
        discharge = parameters[4].valueAsText
        interval = int(parameters[5].valueAsText)
        points = parameters[6].valueAsText

        env.workspace = geodb

        desc = Describe(rivers)
        CreateFeatureclass_management(geodb,points,geometry_type='POINT',spatial_reference=desc.spatialReference)
        fid_name = 'river_ID'
        AddField_management(points, fid_name, 'LONG')

        with da.SearchCursor(rivers, ['SHAPE@', desc.OIDFieldName]) as search_cursor:
            with da.InsertCursor(points, ['SHAPE@', fid_name]) as insert_cursor:
                for row in search_cursor:
                    line = row[0]

                    if line:
                        cur_length = interval
                        max_position = line.length
                        insert_cursor.insertRow([line.firstPoint, row[1]])
                        while cur_length < max_position:
                            insert_cursor.insertRow([line.positionAlongLine(cur_length, False), row[1]])
                            cur_length += interval

        arcpy.AddMessage('Created points')

        sa.ExtractMultiValuesToPoints(points, [[elevation, 'elev']])

        arcpy.AddMessage('Extracted elevation points')

        # calculate the elevation difference between successive points
        AddField_management(points, 'head', 'FLOAT')
        with da.UpdateCursor(points, [fid_name,'elev','head']) as cursor:
            prevFid = 0
            prevElev = 0

            for row in cursor:
                if row[1]: #check to make sure an elevation entry exists
                    currentFid = row[0]
                    currentElev = row[1]

                    if (currentFid == prevFid) and ((prevElev - currentElev) > 0):
                        row[2] = prevElev - currentElev
                    else:
                        row[2] = 0

                    cursor.updateRow(row)
                    prevFid = currentFid
                    prevElev = currentElev
                else:
                    row[2] = 0
                    prevFid = row[0]
                    prevElev = 0

        arcpy.AddMessage('Calculated head')

        # delete all points where head is 0 or negative
        with da.UpdateCursor(points, 'head') as cursor:
            for row in cursor:
                if row[0] <= 0:
                    cursor.deleteRow()

        arcpy.AddMessage('Deleted points with zero head')

        sa.ExtractMultiValuesToPoints(points, [[discharge, 'discharge_second']])

        arcpy.AddMessage('Extracted discharge')

       # before calculating the power, delete any points that will result in a value absurdly large or small (LONG only goes up to 2 billion)
       # currently set at less than 10 litres/sec, or more than 2000 m^3/sec
        dischargeLowerCutoff = 0.01
        dischargeUpperCutoff = 2000
        with da.UpdateCursor(points, 'discharge_second') as cursor:
            for row in cursor:
                if row[0] < dischargeLowerCutoff or row[0] > dischargeUpperCutoff:
                    cursor.deleteRow()

        arcpy.AddMessage('Deleted points with discharge < 0.01 m3/s OR discharge > 2000 m3/s')

        # calculate the power in watts based on Alex's formula
        # P = rho * g * nt * ng * conv * Q * deltaH
        rho = '1000' # density
        g= '9.81' # gravity
        nt = '0.88' # turbine efficiency
        ng = '0.96' # generator efficiency
        conv = '0.6' # conversion factor for environmental flow deduction
        AddField_management(points, 'power', 'FLOAT')
        CalculateField_management(points, 'power', rho+'*'+g+'*'+nt+'*'+ng+'*'+conv+'* !discharge_second! * !head!', 'PYTHON_9.3')

        arcpy.AddMessage('Calculated power')
        arcpy.AddMessage('All done')